% mol3D_main.m
% Numerical simulations of drumlin formation.
% J. S. Fannon, A. C. Fowler, and I. R. Moyles.
% March 2017.

function [P,Q,s_full,b_full,t,dtvec] = mol3D_main()

close all
clear all

%=========================================================================%
%=========================================================================%
% Parameters

% Define time interval vector - code outputs data file at each time stated.
t_int_vec = [1 2];

% Loop over all time intervals.
for rrt = 1:length(t_int_vec)
    
    % For the first time interval we need to define everything.
    if rrt==1
        % Dimensionless parameters apparing in the model.
        alpha_v  = 0.09;
        beta_v   = 1e-2;
        gamma_v  = 5e-3;
        delta_v  = 5e-1;
        lambda_v = 1; % In keeping with approx for A(N) function.
        sigma_v  = 0.64;
        bar_u    = 1;
        ex_power = 3.5; % Power in sediment transport function.
        
        L      = 10; % Domain length/width.
        t_init = 0; % Initial time.
        m      = 100; % Gridpoints in x direction.
        n      = 100; % Gridpoints in y direction.
    else
        % Load in data from last timestep.
        [s0,b0,t_init,alpha_v,gamma_v,delta_v,beta_v,...
        lambda_v,sigma_v,ex_power,bar_u,m,n,L,W,ramp_f]=load_init_data();
    end
     
    % Time iteration parameters.
    tmax = t_int_vec(rrt);
    nout = (20)*(round(10*(tmax-t_init))/10); % Number of outputs from ODE solver.
    if (nout<=0)
        error('Need t_max > t_init')
    end
    if (m)~=(n)
        error('Need same number of gridpoints in each direction')
    end
    
    % Indicator to save data files.
    sdata_ind  = 0;

    % Minres parameters.
    num_it    = 2500; % Number of iterations
    num_it_wg = 2000; % Number of iterations with guess.
    tol_v     = 1e-8; % Tolerance in minres.
    tol_v_wg  = 1e-8;

    % Create grids.
    W = L; % Domain width.
    x = linspace(0,L,m+1);  % Uniform grid in x direction.
    y = linspace(0,W,n+1);  % Uniform grid in y direction.
    dx  = x(2) - x(1); % x grid step.
    dy  = y(2) - y(1); % y grid step.

    % Create cell-centred grid.
    p     = linspace(dx/2,L-(dx/2),m);
    q     = linspace(dy/2,W-(dy/2),n);
    [P,Q] = meshgrid(p,q);

    %=====================================================================%
    %=====================================================================%
    % Initialise temporary matrices.
    % Rows - y direction, columns - x direction.

    % Create sparse matrices relevant for Psi solver.
    e = ones(m,1);
    % Create diagonal matrix.
    d5 = spdiags(e,0,m,m); D5 = kron(speye(n,n),d5);
    % Create sup diagonal matrix.
    sp1 = spdiags(e,1,m,m); sp1(end,1) = 1; % PBC
    D1 = kron(speye(n,n),sp1); 
    % Create sub diagonal matrix.
    sb1 = spdiags(e,-1,m,m); sb1(1,end) = 1;
    D2 = kron(speye(n,n),sb1);
    % Create sup sup diagonal.
    e2 = ones(n,1);
    spp1 = spdiags(e2,1,n,n); spp1(end,1) = 1;
    D4 = kron(spp1,speye(m,m)); 
    % Create sub sub diagonal.
    sbb1 = spdiags(e2,-1,n,n); sbb1(1,end) = 1;
    D3 = kron(sbb1,speye(m,m)); 

    % Create Backward Difference in x.
    XBDIFF = D5 - D2;
    % Create Forward Difference in x.
    XFDIFF = D1 - D5;
    % Create Backward Difference in y.
    YBDIFF = D5 - D3;
    % Create Forward Difference in y.
    YFDIFF = D4 - D5;
    % Create Xplus matrix.
    XPLUS = D5 + D1;
    % Create Xminus matrix.
    XMINUS = D5 + D2;
    % Same for y.
    YPLUS = D5 + D4; YMINUS = D5 + D3;
    % Central Diff matrix (in x).
    XCENDIFF = D1 - D2;
    % Central Diff matrix (in y).
    YCENDIFF = D4 - D3;

    % Create list (used in Psi solver).
    psi_i_y  = 1:(n*m);
    % Create total coefficient matrix.
    J_E = sparse(n*m,n*m);
    % Create the empty source term.
    F_E = zeros(n*m,1);

    % Initial condition for the system.
    if rrt~=1
        % If loading in data from last timestep.
        s_init = s0; b_init = b0;
    else 

        % Create a 2D Gaussian random rough surface.
        [r01,~,~]   = rsgeng2D_mod(m,n,L,W,0.1,max(L,W)/10);
        [r02,~,~]   = rsgeng2D_mod(m,n,L,W,0.1,max(L,W)/10);

        % Base state values for s and b.
        b0 = 0; s0 = b0 + delta_v;          
        ramp_f = 1e-2; % Amplification factor for perturbations.
        
        % Perturb.
        s_init = s0 + ramp_f.*r01;
        b_init = b0 + ramp_f.*r02;
    end

    % Ordered by row i.e. increase along x for fixed y.
    si_col = reshape(s_init',n*m,1);
    bi_col = reshape(b_init',n*m,1);

    % Gather together s and b.
    F = [si_col; bi_col];

    % Initial h.
    h_initcol = (si_col)./(delta_v) - (bi_col./delta_v);

    % Initial Psi.
    [DPsi]=mol3D_psisolve(h_initcol,bi_col,sigma_v,delta_v,lambda_v,...
            dx,dy,D1,D2,D3,D4,D5,psi_i_y,m,n,num_it,tol_v,J_E,F_E); 
    Psi_guess = DPsi;

    % Create spectral stuff.
    n_x = min(((m)/2) - 1,128); n_y = min(((n)/2) - 1,128); 
    k1 = -2*pi*(-n_x:n_x)/L; k1(n_x+1) = 1e-6;  % x wavenumbers.
    k1 = repmat(k1,(n_x+n_y)+1,1);
    k2 = -2*pi*(-n_y:n_y)/W; k2(n_y+1) = 1e-6;  % y wavenumbers.
    k2 = repmat(k2',1,(n_x+n_y)+1);
    k  = sqrt(k1.^2 + k2.^2);
    j  = k./(sigma_v);

    % J hat function.
    J_hat = ((sinh(j)).^2)./(2.*k.*(j+sinh(j).*cosh(j)));

    %=====================================================================%
    %=====================================================================%

    % Time integration.

    F0       = F; % Initial values.
    dtmax    = 1e-2; % Max timestep.
    reltol_v = 1e-6; 
    abstol_v = 1e-8;
    % Options function.
    options  = odeset('Reltol',reltol_v,'AbsTol',abstol_v,'InitialStep',1e-8,...
                'MaxStep',dtmax,'Vectorized','on','Events',@TempEventFcn);                 

    % Number of output intervals.
    tvec    = linspace(t_init,tmax,nout); 
    told    = t_init; % Initial time.
    dtvec   = []; % Vector for timesteps.

    tic
    % Use ode solver
    [t,F]   = ode113(@sb_odevol,tvec,F0,options);
    run_time=toc

    % Reorganise data.
    for i=1:(size(F,1))
        Y_c = F(i,:);
        st            = reshape(Y_c(1:(n*m)),m,n);
        s_full(:,:,i) = st';
        bt            = reshape(Y_c(((n*m)+1):(2*(n*m))),m,n);
        b_full(:,:,i) = bt';
    end

    s_full = s_full(:,:,1:size(F,1));
    b_full = b_full(:,:,1:size(F,1));

    if sdata_ind==1
        qwe = clock;
        filename_1 = sprintf('beta%.3f_gam%.3f_del%.1f_t%.2f_%.2f_cd%d%d%d.mat',...
            beta_v,gamma_v,delta_v,t(1),t(end),qwe(3),qwe(2),qwe(1));
        filename_2 = sprintf('beta%.3f_gam%.3f_del%.1f_t%.2f_cd%d%d%d.mat',...
            beta_v,gamma_v,delta_v,t(end),qwe(3),qwe(2),qwe(1));
        save(filename_1,'s_full','b_full','P','Q','t','alpha_v','beta_v','gamma_v',...
            'delta_v','bar_u','m','n','lambda_v','sigma_v','L','W','dtvec',...
            'ex_power','ramp_f','dtmax','t_init','tmax','reltol_v','abstol_v',...
            'run_time','num_it','num_it_wg','tol_v','tol_v_wg','n_x','n_y')
        save(filename_2,'s','b','P','Q','t','alpha_v','beta_v','gamma_v',...
            'delta_v','bar_u','m','n','lambda_v','sigma_v','L','W','dtvec',...
            'ex_power','ramp_f','dtmax','t_init','tmax','reltol_v','abstol_v',...
            'run_time','num_it','num_it_wg','tol_v','tol_v_wg','n_x','n_y')
    end
    
    filename_3 = sprintf('mol3D_ind_prevdat.mat');
    save(filename_3,'s','b','P','Q','t','alpha_v','beta_v','gamma_v',...
            'delta_v','bar_u','m','n','lambda_v','sigma_v','L','W','dtvec',...
            'ex_power','ramp_f','dtmax','t_init','tmax','reltol_v','abstol_v',...
            'run_time','num_it','num_it_wg','tol_v','tol_v_wg','n_x','n_y')
    
    % If we have had a warning, stop the code.
    if size(F,1)~=nout
        error('Timestep too small')
    end
   
end
    
%=========================================================================%
%=========================================================================%
    
function Fout = sb_odevol(t,F)

    Fout = zeros(2*m*n,size(F,2));
    colnum = size(F,2);
    for jjj=1:colnum
    % Reshape s and b data into columns.
    s = reshape(F(1:(n*m),jjj),m,n); s = s';
    b = reshape(F(((n*m)+1):(2*(n*m)),jjj),m,n); b = b';
    s_col = F(1:(n*m),jjj); b_col = F(((n*m)+1):(2*(n*m)),jjj);
    
    % Find h.
    h     = (s./delta_v) - (b./delta_v);
    h_col = (s_col./delta_v) - (b_col./delta_v);
    
    % Psi solver.
    [D]=mol3D_psisolve_wg(h_col,b_col,sigma_v,delta_v,lambda_v,...
            dx,dy,D1,D2,D3,D4,D5,psi_i_y,m,n,num_it_wg,tol_v_wg,...
            Psi_guess,J_E,F_E);
                
    % Extract.
    Psi_col = D;
    PSI  = reshape(D(psi_i_y),m,n); Psi = PSI';

    % Find tau_e.
    tau_e_x =  - (1./(2*dx))*(h_col).*((XCENDIFF)*Psi_col) + sigma_v*h_col;
    tau_e_y =  - (1./(2*dy))*(h_col).*((YCENDIFF)*Psi_col);
    
    % Solve for all other variables.
    TAUMOD = sqrt(tau_e_x.^2 + tau_e_y.^2);
    B_col = TAUMOD.^(ex_power-1); B = reshape(B_col,m,n); B = B';                  
    % Find N.
    N_col = 1./(lambda_v.*h_col); N = 1./(lambda_v.*h);
    % Find A.
    A_col = AN_fun(N_col); A = AN_fun(N);
    Phi = Psi - s + N;

    % Solve for s using Fourier transform.
    s_hat  = fttrap2(s,n_x,n_y); Phi_hat = fttrap2(Phi,n_x,n_y);

    % Find change in s.
    ds_hat  = (1/alpha_v).*(1i.*k1.*bar_u.*s_hat + J_hat.*Phi_hat);
    % Fourier transform back.
    dsout   = real(ifttrap2(ds_hat,m,n));

    % Find A_x.
    AXT = -1i.*k1.*fttrap2(A,n_x,n_y);
    A_x = real(ifttrap2(AXT,m,n)); A_x = reshape(A_x',m*n,1);

    % Find value of B.
    BH_col = B_col.*h_col; BH = B.*h;

    % Find (Bh)_x.
    BHXT = -1i.*k1.*fttrap2(BH,n_x,n_y);
    Bh_x = real(ifttrap2(BHXT,m,n)); Bh_x = reshape(Bh_x',m*n,1);
        
    % Diffusion terms.
    A3col = A_col.^3;
    A3xp   = (XPLUS)*(A3col);  
    A3xm   = (XMINUS)*A3col; 
    A3yp   = (YPLUS)*(A3col); 
    A3ym   = (YMINUS)*(A3col); 
    Nxp    = (XFDIFF)*(N_col);
    Nxm    = (XBDIFF)*(N_col);
    Nyp    = (YFDIFF)*(N_col);
    Nym    = (YBDIFF)*(N_col); 
    Psi_xp = (XFDIFF)*Psi_col;
    Psi_xm = (XBDIFF)*Psi_col; 
    Psi_yp = (YFDIFF)*Psi_col; 
    Psi_ym = (YBDIFF)*Psi_col;
    BAxp   = (XPLUS)*(BH_col); 
    BAxm   = (XMINUS)*(BH_col); 
    BAyp   = (YPLUS)*(BH_col);
    BAym   = (YMINUS)*(BH_col);

    % Find change in b.
    dbout = -bar_u.*A_x - gamma_v.*sigma_v.*Bh_x + ...
            (beta_v/(2*(dx))).*((A3xp.*Nxp - A3xm.*Nxm)./((dx))) + ...
            (beta_v/(2*(dy))).*((A3yp.*Nyp - A3ym.*Nym)./((dy))) + ...
            (gamma_v/(2*(dx))).*((BAxp.*Psi_xp - BAxm.*Psi_xm)./((dx))) + ...
            (gamma_v/(2*(dy))).*((BAyp.*Psi_yp - BAym.*Psi_ym)./((dy)));    
             
             
    % Output new data.
    dscol = reshape(dsout',n*m,1);
    Fout(:,jjj) = [dscol; dbout];

    if isnan(Fout)
        error('Error in evolution equ\n');
    end

    fprintf('Current time = %g\n',t)
    
    % Update guess.
    Psi_guess = Psi_col;
    end   
end

%=========================================================================%
%=========================================================================%

    function [value,isterminal,direction] = TempEventFcn(t,F)
        % Event function to check timestep size.
        tnew = t;
        dt = abs(tnew - told);
        told = tnew;
        dtvec = [dtvec; dt];
        if (t>(t_init+0.1) && dt<1e-5)
            value = 0;
        else
            value=1;
        end
        isterminal = 1;
        direction = 0;
    end
end
    
    