function [f,x,y] = rsgeng2D_mod(M,N,rL,rW,h,cl,opt)
%
% [f,x,y] = rsgeng2D(M,N,rL,h,cl,opt) 
%
% generates a square isotropic 2-dimensional Gaussian random rough surface 
% f(x,y) with MxN surface points. The surface has a Gaussian height distribution 
% function and Gaussian autocovariance functions (in both x and y), 
% where rL is the length of the surface side, h is the RMS height and cl is 
% the correlation length.
%
% Input:    M   - number of surface points in x
%           N   - number of surface points in y
%           rL  - length of surface in x
%           rW  - length of surface in y
%           h   - rms height
%           cl  - correlation length
%           opt - optional parameter (type 'plot' for plotting surface
%                 profile)
%
% Output:   f  - surface heights
%           x  - surface points
%           y  - surface points
%
% Last updated: 2009-02-23 (David Bergstrm).  

x = linspace(-rL/2,rL/2,M); y = linspace(-rW/2,rW/2,N);

Z = h.*randn(N,M); % uncorrelated Gaussian random rough surface distribution
                   % with mean 0 and standard deviation h
                   
% Gaussian filter
F = zeros(N,M); 
for m = 1:M;
  for n = 1:N;
      F(n,m) = 2/sqrt(pi)*exp(-((x(1,m))^2+(y(1,n))^2)/(cl^2/2));
  end;
end;

%correlation of surface including convolution (faltung), inverse
%Fourier transform and suitable prefactors

f = sqrt((rL*rW)/(N*M))*(1/cl)*ifft2(fft2(Z).*fft2(F));

% optional plotting
if nargin<7 || isempty(opt)
    return;
end;
if nargin==7 
    if ischar(opt)
        if strcmp(opt,'plot');
            mesh(x,y,f);
            xlabel('x')
            ylabel('y')
            zlabel('z=f(x,y)')
            title('Plot of generated surface z=f(x,y)')
        else fprintf('%s is not a valid option. Type \''help rsgeng2D\'' for further details.\n',opt); 
        end;
    else fprintf('Option must be a string. Type \''help rsgeng2D\'' for further details.\n');
    end;
end;
